'''
Created with love by Sigmoid

@Author - Păpăluță Vasile - vpapaluta06@gmail.com
'''
# Importing all needed libraries
import random
from sklearn.metrics import accuracy_score
from sklearn.model_selection import train_test_split
class GeneticAlgorithmSelector:
    def __init__(self, nb_children=4, nb_generation=200, scoring_metric=accuracy_score, max=True):
        '''
            Setting up the model
        :param nb_children: integer, default = 4
            The number of children created at every generation
        :param nb_generation: integer, default = 200
            The number of generations created to find the best feature combination
        :param scoring_metric: function, default = sklearn.metrics.accuracy_score
            The sklearn like scoring metric
        :param max: boolean, default = True
            Used to define whatever we need the biggest of the smallest values computed by self.scoring_metric
        '''
        self.nb_children = nb_children
        self.nb_generation = nb_generation
        self.scoring_metric = scoring_metric
        self.max = max
    def inverse(self, value):
        '''
            This function inverses the value coming to function
        :param value: integer, o or 1
            The value that should be inverted
        :return: integer
            Return the inverse of :param value (if value = 0 ==> 1, else 0)
        '''
        if value == 0:
            return 1
        else:
            return 0
    def cross_over(self, population):
        '''
            This function apply the crossing-over process on 2 arrys-like lists
        :param population: 2-d array like list
            The population with 2 array that will be uses to create new individuals in the population
        :return: 2-d array like list
            Return the population with parents and their children after corssing-over
        '''
        new_generation = []
        for i in range(self.nb_children//2):
            first = random.randrange(0, len(self.X_columns)-1)
            second = random.randrange(0, len(self.X_columns)-1)
            if first > second:
                first, second = second, first
            new_generation.append(population[0][0:first] + population[1][first: second] + population[0][second:])
            new_generation.append(population[1][0:first] + population[0][first: second] + population[1][second:])
        for gene in new_generation:
            population.append(gene)
        return population
    def mutate(self, gene):
        '''
            This function generates a random mutation on a gene
        :param gene: 1-d array like list
            The list with zeros and ones that will be mutated
        :return: 1-drray like list
            The gene list after mutation
        '''
        mutation_locus = random.randrange(0, len(self.X_columns)-1)
        gene[mutation_locus] = self.inverse(gene[mutation_locus])
        return gene
    def gene_to_cols(self, gene):
        '''
            This function convert the zeros and ones list to columns list
        :param gene: 1-d array like list
            The list with zeros and ones that must be transformed in a columns sequence
        :return: 1-d array like list
            The lust with columns that will go to the model
        '''
        cols = []
        for i in range(len(gene)):
            if gene[i] == 1:
                cols.append(self.X_columns[i])
        return cols
    def select(self, algo, dataframe, y_column, test_size=0.2):
        '''
            This function selects the best columns
        :param algo: sklearn algorithm class
            An sklearn algorithm class
        :param dataframe: pandas DataFrame
            Data Frame on which the algorithm is applied
        :param y_column: string
            The column name of the value that we what to predict
        :param test_size: float, default = 0.2
            The percent of data set that will be used to find the trained algorithm accuracy
        :return: list
            The list of columns selected by algorithm
        '''
        self.X_columns = [col for col in dataframe.columns if col != y_column]
        population =[]
        temp = []
        for i in range(len(self.X_columns)):
            temp.append(random.choice([0, 1]))
        population.append(temp)
        temp = [self.inverse(x) for x in population[-1]]
        population.append(temp)
        for gen in range(self.nb_generation):
            print(gen)
            acc = []
            population = self.cross_over(population)
            for i in range(2, len(population)):
                population[i] = self.mutate(population[i])
            for gene in population:
                X = dataframe[self.gene_to_cols(gene)].values
                y = dataframe[y_column].values
                X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=test_size, random_state=0)
                algo.fit(X_train, y_train)
                y_pred = algo.predict(X_test)
                acc.append(self.scoring_metric(y_test, y_pred))
            if self.max:
                res = sorted(range(len(acc)), key=lambda sub: acc[sub])[-2:]
            else:
                res = sorted(range(len(acc)), key=lambda sub: acc[sub])[:2]
            population = [population[res[0]], population[res[1]]]
        return self.gene_to_cols(population[0])


from distutils.core import setup
long_description = '''
Kydavra is a python sci-kit learn inspired package for feature selection. It used some statistical methods to extract from pure pandas Data Frames the columns that are related to column that your model should predict.\n
This version of kydavra has the next methods of feature selection:\n
1) ANOVA test selector (ANOVASelector).\n
2) Chi squared selector (ChiSquaredSelector).\n
3) Genetic Algorithm selector (GeneticAlgorithmSelector).\n
4) Kendall Correlation selector (KendallCorrelationSelector).\n
5) Lasso selector (LassoSelector).\n
6) Pearson Correlation selector (PearsonCorrelationSelector).\n
7) Point-Biserial selector (PointBiserialCorrSelector).\n
8) P-value selector (PValueSelector).\n
9) Spearman Correlation selector (SpermanCorrelationSelector).\n
All these methods takes the pandas Data Frame and y column to select from remained columns in the Data Frame.\n

How to use kydavra\n
To use selector from kydavra you should just import the selector from kydavra in the following framework:\n
```from kydavra import <class name>```\n
class names are written above in parantheses.\n
Next create a object of this algorithm (I will use p-value method as an example).\n
```method = PValueSelector()```\n
To get the best feature on the opinion of the method you should use the 'select' function, using as parameters the pandas Data Frame and the column that you want your model to predict.\n
```selected_columns = method.select(df, 'target')```\n
Returned value is a list of columns selected by the algorithm.\n

Some methods could plot the process of selecting the best features.\n
In these methods dotted are features that wasn't selected by the method.\n
*ChiSquaredSelector*\n
```method.plot_chi2()```\n
For ploting and\n
```method.plot_chi2(save=True, file_path='FILE/PATH.png')```\n
and\n
```method.plot_p_value()```\n
for ploting the p-values.\n
*LassoSelector*\n
```method.plot_process()```\n
also you can save the plot using the same parameters.\n
*PValueSelector*\n
```method.plot_process()```\n

Some advices.\n
1) Use ChiSquaredSelector for categorical features.\n
2) Use LassoSelector and PValueSelector for regression problems.\n
3) Use PointBiserialCorrSelector for binary classification problems.\n

With love from Sigmoid.\n

We are open for feedback. Please send your impression to vpapaluta06@gmail.com\n

'''
setup(
  name = 'kydavra',
  packages = ['kydavra'],
  version = '0.1.2',
  license='MIT',
  description = 'Kydavra is a sci-kit learn inspired python library with feature selection methods for Data Science and Macine Learning Model development',
  long_description=long_description,
  author = 'SigmoidAI - Păpăluță Vasile',
  author_email = 'vpapaluta06@gmail.com',
  url = 'https://github.com/user/reponame',
  download_url = 'https://github.com/ScienceKot/kydavra/archive/v1.0.tar.gz',    # I explain this later on
  keywords = ['ml', 'machine learning', 'feature selection', 'python'],   # Keywords that define your package best
  install_requires=[            # I get to this in a second
          'numpy',
          'pandas',
          'scikit-learn',
          'statsmodels',
          'matplotlib',
          'seaborn'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Framework :: Jupyter',
    'Intended Audience :: Science/Research',
    'Topic :: Scientific/Engineering :: Artificial Intelligence',
    'Topic :: Scientific/Engineering :: Mathematics',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
  ],
)


from kydavra.ANOVASelector import ANOVASelector
from kydavra.ChiSquaredSelector import ChiSquaredSelector
from kydavra.KendallCorrelationSelector import KendallCorrelationSelector
from kydavra.LassoSelector import LassoSelector
from kydavra.PearsonCorrelationSelector import PearsonCorrelationSelector
from kydavra.PointBiserialSelector import PointBiserialCorrSelector
from kydavra.PValueSelector import PValueSelector
from kydavra.SpearmanCorrelationSelector import SpermanCorrelationSelector
from kydavra.GeneticAlgorithmSelector import GeneticAlgorithmSelector